﻿Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

Imports Autodesk.AutoCAD.Colors

Imports Autodesk.AutoCAD.Windows

Imports System.Windows.Controls

<Assembly: CommandClass(GetType(dotNetTraining_Webcast_Demos.Webcast_6_Demos))> 

Namespace dotNetTraining_Webcast_Demos
    Public Class Webcast_6_Demos


        ' Global for the user control
        Public m_UserCtrl As UserControl1

        <CommandMethod("DemoPointmonitor")> _
        Public Sub pointMonitorDemo()

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
            ' set a system variable
            Application.SetSystemVariable("ROLLOVERTIPS", 0)

            ' make sure palette is created
            If dotNetTraining_Webcast_Demos.Webcast_4_Demos.m_PaletteSet Is Nothing Then

                ' tell the user to run the command
                ed.WriteMessage(vbCr + "Run the DemoPalette command first")

                Return
            End If

            ' get User control instantiated
            m_UserCtrl = dotNetTraining_Webcast_Demos.Webcast_4_Demos.m_UserCtrl

            ' Add point monitor event
            AddHandler ed.PointMonitor, New PointMonitorEventHandler(AddressOf thePointMonitor)
           

        End Sub


        Public Sub thePointMonitor(ByVal sender As Object, ByVal e As PointMonitorEventArgs)
            ' get entities
            Dim fullEntPath() As FullSubentityPath = e.Context.GetPickedEntities()
            ' ensure entities are under the cursor
            If (fullEntPath.Length) Then
                ' start transaction
                Dim trans As Transaction = _
                 Application.DocumentManager.MdiActiveDocument.Database.TransactionManager.StartTransaction

                Try
                    ' Open the entity under the cursor
                    Dim ent As Entity = trans.GetObject(fullEntPath(0).GetObjectIds(0), OpenMode.ForRead)
                    ' addd a tooltip
                    e.AppendToolTipText("The Entity is a " + ent.GetType().ToString())
                    ' use for test
                    Dim bItemFound As Boolean = False
                    ' use for a test
                    Dim strTest As String

                    Dim strLBItem As String



                    ' going through each of the items in the list box
                    For Each strLBItem In m_UserCtrl.ListBox1.Items
                        
                        strTest = "ObjectId of Entity = " & " " & ent.ObjectId.ToString()

                        If strLBItem = strTest Then

                            bItemFound = True
                            ' toggle the color of entries in the list
                            If m_UserCtrl.ListBox1.ForeColor = Drawing.Color.Black Then

                                m_UserCtrl.ListBox1.ForeColor = Drawing.Color.Aqua

                            Else

                                m_UserCtrl.ListBox1.ForeColor = Drawing.Color.Black

                            End If


                            Exit For
                        End If
                    Next

                    ' Add the entry in list box if it is not there
                    If (Not bItemFound) Then

                        m_UserCtrl.ListBox1.Items.Add("ObjectId of Entity = " & " " & ent.ObjectId.ToString())

                    End If

                    trans.Commit()
                Catch ex As System.Exception


                    Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(ex.Message)

                Finally

                    trans.Dispose()

                End Try

            End If

        End Sub

        <CommandMethod("DemoMonitorWith_A_getPoint")> _
        Public Sub monitorWithGetPointDemo()


            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
            ' Add the point monitor event
            AddHandler ed.PointMonitor, New PointMonitorEventHandler(AddressOf thePointMonitor_2)



            ed.TurnForcedPickOn()


            Dim getPointOptions As PromptPointOptions = New PromptPointOptions("Pick A Point : ")


            Dim getPointResult As PromptPointResult = ed.GetPoint(getPointOptions)

            ' if ok
            'If (getPointResult.Status = PromptStatus.OK) Then
            '    ' do something...
            'End If

            RemoveHandler ed.PointMonitor, AddressOf thePointMonitor_2

        End Sub

        Public Sub thePointMonitor_2(ByVal sender As Object, ByVal e As PointMonitorEventArgs)

            ' first lets check what is under the Cursor
            Dim fullEntPath() As FullSubentityPath = e.Context.GetPickedEntities()

            If (fullEntPath.Length) Then


                Dim trans As Transaction = _
                  Application.DocumentManager.MdiActiveDocument.Database.TransactionManager.StartTransaction

                Try

                    Dim ent As Curve = trans.GetObject(fullEntPath(0).GetObjectIds(0), OpenMode.ForRead)

                    '  make sure entity has extenstion dictionary
                    If (ent.ExtensionDictionary.IsValid) Then

                        ' get the dictionary
                        Dim extensionDict As DBDictionary = trans.GetObject(ent.ExtensionDictionary, OpenMode.ForRead)

                        ' get ObjectId
                        Dim entryId As ObjectId = extensionDict.GetAt("CustomData")


                        Dim myXrecord As Xrecord
                        ' get data
                        myXrecord = trans.GetObject(entryId, OpenMode.ForRead)


                        For Each myTypedVal As TypedValue In myXrecord.Data

                            ' check to see if the data is a real, then use it
                            If (myTypedVal.TypeCode = DxfCode.Real) Then


                                Dim pnt As Point3d = ent.GetPointAtDist(myTypedVal.Value)
                                ' get the size of pixels in the view
                                Dim pixels As Point2d = e.Context.DrawContext.Viewport.GetNumPixelsInUnitSquare(pnt)

                                ' get x value
                                Dim xDist As Double = 10 / pixels.X
                                ' get the y values
                                Dim yDist As Double = 10 / pixels.Y

                                ' create a temporary circle
                                Dim circle As Circle = New Circle(pnt, Vector3d.ZAxis, xDist)
                                ' pass into the draw method
                                e.Context.DrawContext.Geometry.Draw(circle)
                                ' create temporay text
                                Dim text As DBText = New DBText
                                ' make settings of the text the current settings
                                text.SetDatabaseDefaults()

                                text.Position = pnt + New Vector3d(xDist, yDist, 0)
                                ' set the text
                                text.TextString = myTypedVal.Value.ToString()

                                text.Height = yDist

                                text.ColorIndex = 4

                                e.Context.DrawContext.Geometry.Draw(text)


                            End If
                        Next
                    End If

                    ' all ok, commit it
                    trans.Commit()
                Catch ex As System.Exception

                    Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
                    ed.WriteMessage(ex.ToString())

                Finally
                    ' whatever happens we must dispose the transaction
                    trans.Dispose()

                End Try

            End If

        End Sub

    End Class
End Namespace
